from . import HobokenTestCase, skip, hoboken
import os
import yaml
import unittest
from .helpers import parametrize, parameters
from webob import Request

Matcher = hoboken.matchers.HobokenRouteMatcher


class TestMethods(HobokenTestCase):
    def after_setup(self):
        for meth in self.app.SUPPORTED_METHODS:
            self.app.add_route(meth, '/', self.body_func)

    def test_successful_methods(self):
        methods = list(self.app.SUPPORTED_METHODS)
        methods.remove("HEAD")
        for meth in methods:
            self.assert_body_is('request body', method=meth)

    def test_HEAD_method(self):
        self.assert_body_is('', method="HEAD")

    def test_failed_methods(self):
        for meth in self.app.SUPPORTED_METHODS:
            self.assert_not_found(path='/somebadpath')


class TestHeadFallback(HobokenTestCase):
    def after_setup(self):
        @self.app.get('/')
        def get_func():
            self.app.response.headers['X-Custom-Header'] = 'foobar'
            return 'get body'

    def test_HEAD_fallback(self):
        r = Request.blank('/')
        r.method = "HEAD"
        resp = r.get_response(self.app)

        self.assert_equal(resp.status_int, 200)
        self.assert_equal(len(resp.body), 0)
        self.assert_equal(resp.headers['X-Custom-Header'], 'foobar')


# Load our list of test cases from our yaml file.
curr_dir = os.path.abspath(os.path.dirname(__file__))
test_file = os.path.join(curr_dir, 'routing_tests.yaml')
with open(test_file, 'rb') as f:
    file_data = f.read()
test_cases = list(yaml.load_all(file_data))

def make_name(idx, param):
    return "test_" + param['name']

@parametrize
class TestRouting(HobokenTestCase):
    @parameters(test_cases, name_func=make_name)
    def test_route(self, param):
        if 'skip' in param:
            if hasattr(unittest, 'SkipTest'):
                raise unittest.SkipTest(param['skip'])
            return

        matcher = Matcher(param['path'])
        regex = param['regex']
        self.assert_equal(matcher.match_re.pattern, regex)

        class FakeRequest(object):
            path = None

        for succ in param.get('successes', []):
            r = FakeRequest()
            r.path = succ['route']
            matched, args, kwargs = matcher.match(r)

            expected_args = succ.get('args', [])
            expected_kwargs = succ.get('kwargs', {})
            self.assert_equal(matched, True)
            self.assert_equal(args, expected_args)
            self.assert_equal(kwargs, expected_kwargs)

        for fail in param.get('failures', []):
            r = FakeRequest()
            r.path = fail
            matched, _, _ = matcher.match(r)

            self.assert_equal(matched, False)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMethods))
    suite.addTest(unittest.makeSuite(TestHeadFallback))
    suite.addTest(unittest.makeSuite(TestRouting))

    return suite




from __future__ import with_statement, print_function

from .helpers import *

import os
import unittest
from webob import Request

ensure_in_path(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
import hoboken


class HobokenTestCase(BaseTestCase):
    """
    This is a testcase for Hoboken that contains helpful functions specifically for Hoboken
    """
    def setup(self):
        # We create an application for each test.
        self.app = hoboken.HobokenApplication(self.__class__.__name__)
        self.after_setup()

    def after_setup(self):
        """
        This is a hook for post-setup initialization.  Useful for e.g. creating routes
        """
        pass

    def body_func(self):
        """
        This is a simple function that just returns some data, sufficient for testing a route
        """
        return 'request body'

    def call_app(self, path='/', method='GET', user_agent=None, host=None, accepts=None):
        """
        This function calls our application, and returns a tuple of (status, body)
        """
        req = Request.blank(path)
        req.method = method
        if user_agent is not None:
            req.headers['User-Agent'] = user_agent

        if host is not None:
            req.host = host

        if accepts is not None:
            req.accept = accepts

        resp = req.get_response(self.app)
        return resp.status_int, resp.text

    def assert_body_is(self, body, *args, **kwargs):
        """
        This is a helper function for the most common type of test: calling an application,
        and verifying that it succeeded and that the body matches a given set of data.
        """
        status, data = self.call_app(*args, **kwargs)
        self.assert_equal(status, 200)
        self.assert_equal(data, body)

    def assert_not_found(self, *args, **kwargs):
        """
        This is a helper function that simply asserts that a given request is not found
        """
        status, _ = self.call_app(*args, **kwargs)
        self.assert_equal(status, 404)


def suite():
    # Import test suites here.
    from .test_HobokenApplication import suite as suite_1
    from .test_matchers import suite as suite_2
    from .test_beforeafter import suite as suite_3
    from .test_conditions import suite as suite_4
    from .test_helpers import suite as suite_5
    from .test_routing import suite as suite_6
    from .test_request_response import suite as suite_7

    suite = unittest.TestSuite()
    suite.addTest(suite_1())
    suite.addTest(suite_2())
    suite.addTest(suite_3())
    suite.addTest(suite_4())
    suite.addTest(suite_5())
    suite.addTest(suite_6())
    suite.addTest(suite_7())

    return suite


def main():
    """
    This runs the our tests, suitable for a command-line application
    """
    try:
        unittest.main(defaultTest='suite')
    except Exception as e:
        print("Exception: {0!s}".format(e))

if __name__ == "__main__":
    main()




from __future__ import print_function
import re
from collections import Iterable
from webob.acceptparse import MIMEAccept

def user_agent(match_re):
    regex = re.compile(match_re)

    def user_agent_func(req):
        ua = req.headers['User-Agent']
        if regex.match(ua):
            return True
        else:
            return False

    return user_agent_func


def host(match_re):
    regex = re.compile(match_re)

    def host_func(req):
        return regex.match(req.host)

    return host_func


def accepts(mimetypes):
    # Note that we can't simply check for an iterable here, since
    # strings are also iterables.
    if isinstance(mimetypes, list):
        will_accept = mimetypes
    else:
        will_accept = [mimetypes]

    def accepts_func(req):
        for a in will_accept:
            # print('matching {0} against {1}'.format(str(req.accept), str(a)))
            #if str(req.accept) in a:
            if a in req.accept:
                return True

        return False

    return accepts_func




# Future-proofing
from __future__ import with_statement, absolute_import

# Stdlib dependencies
import re
import sys

# In-package dependencies
from .exceptions import *
from .compat import *


class AbstractMatcher(object):
    """
    This class is the abstract class for all matchers that are used by Hoboken.
    """
    def match(self, request):
        """
        This function performs matching against a given Request.  If the
        request matches, then this function will return (True, args, kwargs),
        where args and kwargs are optional parameters to be passed to the route
        function.  If the request does not match, then this function must
        return (False, _, _), where the underscores are any value.
        """
        raise NotImplementedError("match() is not implemented in the base class")

    def reverse(self, args, kwargs):
        """
        This function provides support for reversing - i.e. given arguments,
        return the associate path.  This function must either return a string
        representing the reversed path, or None if the route cannot be reversed
        (either because there's no support for reversal, or because the passed
        parameters were invalid).
        """
        raise NotImplementedError("reverse() is not implemented in the base class")


class BasicMatcher(AbstractMatcher):
    """
    Basic matcher - just checks if the path matches exactly.
    """
    def __init__(self, path, case_sensitive=True):
        self.path = path
        self.case_sensitive = case_sensitive

    def match(self, request):
        if self.case_sensitive:
            matches = self.path == request.path
        else:
            matches = self.path.lower() == request.path.lower()

        # No arguments, so always return empty values.
        return matches, [], {}

    def reverse(self, args, kwargs):
        # Reversing a basic route is just the path itself.
        return self.path

    def __str__(self):
        return self.path

    def __repr__(self):
        return "BasicMatcher(path={0!s}, case_sensitive={1}".format(self.path, self.case_sensitive)


class RegexMatcher(AbstractMatcher):
    """
    This class matches a URL using a provided regex.
    """

    def __init__(self, regex, key_types, key_names):
        # We handle regexes and string patterns for regexes here.
        if isinstance(regex, RegexType):
            self.re = regex
        elif isinstance(regex, BaseStringType):     # pragma: no cover
            try:
                self.re = re.compile(regex)
            except re.error:
                raise TypeError("Parameter 'regex' is not a valid regex")
        else:                                       # pragma: no cover
            raise TypeError("Parameter 'regex' is not a valid regex")

        # Save keys.
        self.key_types = key_types
        self.key_names = key_names

    def match(self, request):
        match = self.re.match(request.path)
        args = []
        kwargs = {}

        if match:
            matches = match.groups()

            # Merge in the captures.
            for t, k, v in zip(self.key_types, self.key_names, matches):
                # Depending on the type, either add to urlargs or urlparams.
                if t == True:
                    if k is not None:
                        kwargs[k] = v
                else:
                    args.append(v)

            # We save all captures in the special parameter "_captures".
            kwargs["_captures"] = list(matches)

        return match, args, kwargs

    def reverse(self, args, kwargs):
        # We cannot reverse a regex.
        return False

    def __str__(self):
        return self.re.pattern

    def __repr__(self):
        return "RegexMatcher(regex={0!r}, key_types={1!r}, key_names={2!r})".format(self.re.pattern, self.key_types, self.key_names)


class HobokenRouteMatcher(AbstractMatcher):
    # These regexes mirror those of Ruby's URI module.
    ALPHANUMERIC = b"A-Za-z0-9"
    UNRESERVED = br"\-_.!~*'()" + ALPHANUMERIC
    RESERVED = Br";/?:@&=+$,\[\]"
    UNSAFE = br"[^" + UNRESERVED + RESERVED + b"]"
    unsafe_re = re.compile(UNSAFE)

    # This regex is what we use to find params/splats.
    MATCH_REGEX_PATTERN = r'((:\w+)|\*)'
    MATCH_REGEX_BYTES = re.compile(MATCH_REGEX_PATTERN.encode('latin-1'))
    MATCH_REGEX_STR = re.compile(MATCH_REGEX_PATTERN)

    def __init__(self, route):
        match_regex = self._convert_path(route)
        self.match_re = re.compile(match_regex)

    def _save_fragments(self, match):
        """
        Store the fragments between matches, so that we can rebuild the path,
        given a route.
        """
        self.fragments = []
        last_fragment = 0
        for m in self.MATCH_REGEX_STR.finditer(match):
            frag_start = last_fragment
            frag_end = m.start(0)
            frag_content = match[frag_start:frag_end]
            self.fragments.append(frag_content)

            last_fragment = m.end(0)

        # Add the final fragment (or a blank string).
        if last_fragment != len(match):
            self.fragments.append(match[last_fragment:])
        else:
            self.fragments.append('')

    def _url_encode_char(self, c):
        c = hex(ord(c))[2:].upper()
        c = b'%' + c.encode('ascii').zfill(2)
        return c

    def _url_encode(self, c):
        if self.unsafe_re.match(c) is not None:
            c = self._url_encode_char(c)
        return c

    def _convert_path(self, path):
        """
        This function will convert a Hoboken-style route path into a regex, and
        also save enough information that we can reverse this path, given some
        args and kwargs.
        """

        # Before we do anything else, save the fragments of the path so we can
        # reconstruct it.
        self._save_fragments(path)

        # We need to extract the splats, the named parameters, and then create
        # a regex to match it.
        #
        # The general rules are as follows:
        #  - Block parameters like :block match one path segment -
        #    i.e. until the next "/", special character, or end-of-
        #    string.  Note that blocks must match at least one
        #    character.
        #  - Splats match anything, but always match non-greedily.
        #    Splats can also match the empty string (i.e. nothing).
        #
        # So, we convert to a regex in the following way:
        #  - Blocks are converted like this:
        #      blah:block --> r"blah([^/?#]+)"
        #  - Splats are converted like this:
        #      blah*blah  --> r"blah(.*?)blah"

        # This class gets around the lack of the nonlocal keyword on python 2.X
        class Store(object):
            ignore = b""
            num_groups = 0

        # This function escapes a character with regex encoding and url-encoding.
        def escaped(c):
            return [re.escape(x) for x in [c, self._url_encode_char(c)]]

        # This function will return a regex that matches a given character in
        # any of the ways it might appear in a URL.  Note that we special-case
        # spaces to also match plus signs.
        def encoded(c):
            # We handle the case of Space specifically.
            if c == b' ':
                return b'(?:\%20' + b'|' + encoded(b'+') + b')'

            # If the encoding doesn't change, we match both the encoded and un-
            # encoded version.  Otherwise, we match just the encoded version.
            # Note that we regex-escape in all cases.
            char = self._url_encode(c)
            if char == c:
                char = b'(?:' + b'|'.join(escaped(c)) + b')'
            else:
                char = re.escape(char)

            return char

        # This function will convert a matched character into the proper encoding.
        def encode_character(match):
            char = match.group(0)
            if char == b'.' or char == b'@':
                Store.ignore += b''.join(escaped(char))
            return encoded(char)

        # This dict stores group names - 'None' for a splat, otherwise the name
        # of the param.
        self.group_names = {}

        def convert_match(match):
            # Get the group number.
            group_num = Store.num_groups
            Store.num_groups += 1

            # Return the appropriate regex.
            if match.group(0) == b'*':
                self.group_names[group_num] = None
                return br"(.*?)"
            else:
                self.group_names[group_num] = match.group(0)[1:].decode('ascii')
                return br"([^" + Store.ignore + br"/?#]+)"

        # Before doing anything else, we need to deal with encoding.  On Python
        # 2, we take the input as-is if it's a str, or encode to utf-8 if it's
        # a unicode value.  On Python 3, we encode it as bytes (the input must
        # be a string on Python 3) using utf-8.
        if sys.version_info[0] >= 3:        # pragma: no cover
            path = path.encode('utf-8')
        else:                               # pragma: no cover
            if isinstance(path, unicode):
                path = path.encode('utf-8')

        pattern = re.sub(br'[^\?\%\\\/\:\*\w]', encode_character, path)
        pattern = self.MATCH_REGEX_BYTES.sub(convert_match, pattern)
        pattern = br'\A' + pattern + br'\Z'

        return pattern.decode('ascii')

    def match(self, request):
        match = self.match_re.match(request.path)
        args = []
        kwargs = {}

        does_match = (match is not None)
        if does_match:
            groups = list(match.groups())
            for idx in range(len(groups)):
                group_name = self.group_names[idx]
                if group_name is None:
                    args.append(groups[idx])
                else:
                    kwargs[group_name] = groups[idx]

        return does_match, args, kwargs

    def reverse(self, args, kwargs):
        num_fragments = len(self.fragments)
        final_route = ""
        args_index = 0

        # For each fragment, we append it, and then lookup the associated type
        # for the current group.  If it's an arg, we grab the current arg and
        # then increment the arg count, otherwise we grab the right kwarg.
        for i in range(num_fragments - 1):
            final_route += self.fragments[i]

            group_name = self.group_names[i]
            if group_name is None:
                final_route += args[args_index]
                args_index += 1
            else:
                value = kwargs[group_name] or ''
                final_route += value

        return final_route + self.fragments[-1]




# -*- coding: utf-8 -*-

from . import HobokenTestCase, BaseTestCase
from ..matchers import BasicMatcher, RegexMatcher, HobokenRouteMatcher
from .helpers import parametrize, parameters

import re
import os
import sys
import yaml
import unittest
from mock import patch, MagicMock

class TestBasicMatcher(BaseTestCase):
    def test_basic_matcher(self):
        m = BasicMatcher('/foobar')
        req = MagicMock(path="/foobar")

        matches, _, _ = m.match(req)
        self.assert_true(matches)

    def test_basic_matcher_case_insensitive(self):
        m = BasicMatcher('/foobar', case_sensitive=False)
        req = MagicMock(path="/FOOBAR")

        matches, _, _ = m.match(req)
        self.assert_true(matches)

    def test_will_reverse(self):
        m = BasicMatcher('/foobar')
        self.assert_equal(m.reverse(None, None), '/foobar')


class TestRegexMatcher(BaseTestCase):
    def test_compiles(self):
        with patch('re.compile') as mock_obj:
            mock_obj.return_value = 0

            test_re = "test_string"
            r = RegexMatcher(test_re, [], [])

            self.assert_is_instance(r, RegexMatcher)
            mock_obj.assert_called_with(test_re)

    def test_raises_on_invalid(self):
        with self.assert_raises(TypeError):
            _ = RegexMatcher("asdf.(")

        with self.assert_raises(TypeError):
            _ = RegexMatcher(123)

    def test_fills_keys(self):
        test_re = "blah(.*?)middle(.*?)end"
        test_data = "blahFOOmiddleBARend"

        m = RegexMatcher(test_re, [True, True], ['one', 'two'])
        req = MagicMock(path=test_data)

        matches, args, kwargs = m.match(req)
        self.assert_true(matches)
        self.assert_in('one', kwargs)
        self.assert_in('two', kwargs)

        self.assert_equal(kwargs['one'], 'FOO')
        self.assert_equal(kwargs['two'], 'BAR')

        self.assert_in('_captures', kwargs)
        self.assert_equal(kwargs['_captures'], ['FOO', 'BAR'])

    def test_with_existing_regex(self):
        r = re.compile("/foob(.*?)ar")
        m = RegexMatcher(r, [False], [None])

        req = MagicMock(path='/foobAAAAAAAAAAAAAar')

        matches, args, _  = m.match(req)
        self.assert_true(matches)
        self.assert_equal(args, ['AAAAAAAAAAAAA'])

    def test_reverse(self):
        m = RegexMatcher("/asdf", [], [])
        self.assert_false(m.reverse(None, None))

# Load our list of test cases from our yaml file.
curr_dir = os.path.abspath(os.path.dirname(__file__))
test_file = os.path.join(curr_dir, 'reversing_tests.yaml')
with open(test_file, 'rb') as f:
    file_data = f.read()
test_cases = list(yaml.load_all(file_data))

def make_name(idx, param):
    return "test_" + param['name']

@parametrize
class TestHobokenRouteMatcher(BaseTestCase):
    @parameters(test_cases, name_func=make_name)
    def test_handles_unicode(self, param):
        path = param['path']
        args = param.get('args', [])
        kwargs = param.get('kwargs', {})

        m = HobokenRouteMatcher(path)
        out = m.reverse(args, kwargs)

        self.assert_equal(out, param['reverse'])

    def test_reversing(self):
        if sys.version_info[0] >= 3:
            return

        unicode_str = b'/f\xc3\xb8\xc3\xb8'.decode('utf-8')

        m = HobokenRouteMatcher(unicode_str)
        request = MagicMock(path="/f%C3%B8%C3%B8")
        matches, _, _ = m.match(request)
        self.assert_true(matches)


def suite():
    suite = unittest.TestSuite()

    suite.addTest(unittest.makeSuite(TestBasicMatcher))
    suite.addTest(unittest.makeSuite(TestRegexMatcher))
    suite.addTest(unittest.makeSuite(TestHobokenRouteMatcher))

    return suite




from __future__ import absolute_import

# This is the canonical package information.
__version__ = '0.1.0'
__author__  = 'Andrew Dunham'
__license__ = 'Apache'
__copyright__ = "Copyright (c) 2012, Andrew Dunham"

# Imports we import into the namespace.
from .application import HobokenBaseApplication, condition, halt, \
        pass_route

# Submodules we pull in here.
from . import matchers
from . import exceptions

# Grab mixins.
from .helpers import *

# Build our actual application.
class HobokenApplication(HobokenBaseApplication, HobokenCachingMixin, HobokenRedirectMixin):
    pass



# -*- coding: utf-8 -*-

from . import HobokenTestCase
from ..application import Request, Response
import unittest


class TestRequest(HobokenTestCase):
    def test_is_safe(self):
        for method in ['GET', 'HEAD', 'OPTIONS', 'TRACE']:
            r = Request.blank("/", method=method)
            self.assert_true(r.is_safe)

    def test_is_idempotent(self):
        for method in ['PUT', 'DELETE']:
            r = Request.blank("/", method=method)
            self.assert_true(r.is_idempotent)


class TestResponse(HobokenTestCase):
    def test_is_informational(self):
        e = Response(status_int=100)
        self.assert_true(e.is_informational)

    def test_is_success(self):
        e = Response(status_int=200)
        self.assert_true(e.is_success)

    def test_is_redirect(self):
        e = Response(status_int=300)
        self.assert_true(e.is_redirect)

    def test_is_client_error(self):
        e = Response(status_int=400)
        self.assert_true(e.is_client_error)

    def test_is_server_error(self):
        e = Response(status_int=500)
        self.assert_true(e.is_server_error)

    def test_is_not_found(self):
        e = Response(status_int=404)
        self.assert_true(e.is_not_found)



def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestRequest))
    suite.addTest(unittest.makeSuite(TestResponse))

    return suite



# Future-proofing
from __future__ import with_statement, absolute_import, print_function

# Stdlib dependencies
import sys
import re
import urllib
import logging
import traceback
try:
    import threading
except:                     # pragma: no cover
    import dummy_threading as threading
#from functools import wraps

# External dependencies
import webob
from webob.exc import HTTPMethodNotAllowed, HTTPNotFound, HTTPInternalServerError

# In-package dependencies
from .exceptions import *
from .matchers import *
from .compat import *


def get_func_attr(func, attr, default=None, delete=False):      # pragma: no cover
    if sys.version_info[0] >= 3:
        func_dict = func.__dict__
    else:
        func_dict = func.func_dict

    if delete:
        return func_dict.pop(attr, default)
    else:
        return func_dict.get(attr, default)

def set_func_attr(func, attr, value):                           # pragma: no cover
    if sys.version_info[0] >= 3:
        func_dict = func.__dict__
    else:
        func_dict = func.func_dict

    func_dict[attr] = value


class Request(webob.BaseRequest):
    """
    Hoboken's request objects.  Uses WebOb's request object
    """
    @property
    def is_safe(self):
        """
        Returns True if the request is considered "safe" - i.e. if the request
        should be treated as not modifying any state.
        """
        return self.method in ['GET', 'HEAD', 'OPTIONS', 'TRACE']

    @property
    def is_idempotent(self):
        """
        Returns True if the request is considered idempotent - i.e. if two
        successive identical requests should result in the same result.
        """
        return self.is_safe or self.method in ['PUT', 'DELETE']

class Response(webob.Response):
    """
    Hoboken's request objects.  Uses WebOb's request object.
    """
    @property
    def is_informational(self):
        return 100 <= self.status_int <= 199

    @property
    def is_success(self):
        return 200 <= self.status_int <= 299

    @property
    def is_redirect(self):
        return 300 <= self.status_int <= 399

    @property
    def is_client_error(self):
        return 400 <= self.status_int <= 499

    @property
    def is_server_error(self):
        return 500 <= self.status_int <= 599

    @property
    def is_not_found(self):
        return self.status_int == 404


def condition(condition_func):
    def internal_decorator(func):
        # Either call the add_condition() func, or add this condition to the
        # list of conditions on the function.
        add_condition = get_func_attr(func, 'hoboken.add_condition')
        if add_condition is not None:
            add_condition(condition_func)
            return func

        conditions_arr = get_func_attr(func, 'hoboken.conditions', default=[])
        conditions_arr.append(condition_func)
        set_func_attr(func, 'hoboken.conditions', conditions_arr)

        return func

    return internal_decorator


def halt(*args, **kwargs):
    """
    This function halts routing, and returns immediately.  If the code or body
    parameters are given, this will set those values on the response.
    """
    raise HaltRoutingException(*args, **kwargs)


def pass_route():
    """
    This function signals that we should stop processing the current route and
    continue trying to match routes.  If no more routes are found, a 404 error
    will be returned.
    """
    raise ContinueRoutingException()



class Route(object):
    """
    This class is an abstraction around a URL route.  It encapsulates:
      - The request method.
      - Any conditions defined for the route.
      - A matcher that determines if the route matches a request, and also
        returns any parameters from the request.
      - And finally, the route function itself.
    """
    def __init__(self, matcher, func, conditions=None):
        self.matcher = matcher
        self.func = func
        self.conditions = conditions or []

        self._method = None

    @property
    def method(self):
        return self._method

    @method.setter
    def method(self, val):
        self._method = val.upper()

    def add_condition(self, condition):
        self.conditions.append(condition)

    def reverse(self, *args, **kwargs):
        return self.matcher.reverse(args, kwargs)

    def __call__(self, request, response):
        """
        Call this route.  This function will return True or False, depending on
        whether the route matched and was processed.  It will also catch any
        ContinueRoutingExceptions and return False.
        """
        # self.logger.debug("Processing route: {0}".format(repr(route_tuple)))

        # Reset the parameters in the request before each match,
        request.urlargs = ()
        request.urlvars = {}

        # Do the match.
        does_match, args, kwargs = self.matcher.match(request)
        if not does_match:
            return False, None
        else:
            request.urlargs = tuple(args)
            request.urlvars = kwargs

        try:
            for cond in self.conditions:
                if not cond(request):
                    raise ContinueRoutingException

            # We remove the optional "_captures" kwarg, if it exists.
            kwargs.pop('_captures', None)
            ret = self.func(*args, **kwargs)

        except ContinueRoutingException:
            return False, None

        return True, ret



class HobokenMetaclass(type):
    """
    This class does "black magic" to create an instance of HobokenApplication
    by dynamically adding methods to the class that handle each of the methods
    that are listed in SUPPORTED_METHODS.
    """

    def __new__(klass, name, bases, attrs):
        # This function gets around Python's scoping of lambdas.
        def lambda_factory(method):
            return lambda self, match: self._decorate_and_route(method, match)

        # Grab the SUPPORTED_METHODS constant and use this to dynamically add methods.
        for method in attrs.get('SUPPORTED_METHODS', []):
            new_func = lambda_factory(method)
            new_func.__name__ = method.lower()
            attrs[method.lower()] = new_func

        # Call the base's __new__ now with our modified attributes.
        return super(HobokenMetaclass, klass).__new__(klass, name, bases, attrs)


def with_metaclass(meta, base=object):
    """Create a base class with a metaclass."""
    return meta("NewBase", (base,), {})


def is_route(func):
    return get_func_attr(func, 'hoboken.route', default=False)


class HobokenBaseApplication(with_metaclass(HobokenMetaclass)):
    __metaclass__ = HobokenMetaclass

    # These are the supported HTTP methods.  They can be overridden in
    # subclasses to add additional methods (e.g. "TRACE", "CONNECT", etc.)
    SUPPORTED_METHODS = ("GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS", "HEAD")

    def __init__(self, name, sub_app=None, debug=False):
        self.name = name
        self.sub_app = sub_app
        self.debug = debug

        # Routes array. We split this by method, both for speed and simplicity.
        self.routes = {}
        # TODO: Might be worth having a reverse-mapping array, so we can turn a
        # function into a route (e.g. for URL generation, and so on).

        for m in self.SUPPORTED_METHODS:
            self.routes[m] = []

        # Before and after filter arrays.  Note that these are also Routes
        self.before_filters = []
        self.after_filters = []

        # Create logger.
        self.logger = logging.getLogger("hoboken.applications." + self.name)

        # Configure logger.
        formatter = logging.Formatter('[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d] %(message)s')
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)

        if self.debug:
            self.logger.setLevel(logging.DEBUG)
            handler.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.WARN)
            handler.setLevel(logging.WARN)

        # Set up threadlocal storage.  We use this so we can process multiple
        # requests at the same time from one app.
        self._locals = threading.local()
        self._locals.request = None
        self._locals.response = None

    @property
    def request(self):
        return self._locals.request

    @request.setter
    def request(self, val):
        self._locals.request = val

    @request.deleter
    def request(self):
        self._locals.request = None

    @property
    def response(self):
        return self._locals.response

    @response.setter
    def response(self, val):
        self._locals.response = val

    @response.deleter
    def response(self):
        self._locals.response = None

    def set_subapp(self, subapp):
        self.sub_app = subapp

    def _make_route(self, match, func):
        if isinstance(match, BaseStringType):
            matcher = HobokenRouteMatcher(match)
        elif isinstance(match, RegexType):
            # match is a regex, so we extract any named groups.
            keys = [None] * match.groups
            types = [False] * match.groups
            for name, index in match.groupindex.items():
                types[index - 1] = True
                keys[index - 1] = name

            # Append the route with these keys.
            matcher = RegexMatcher(match, types, keys)

        elif hasattr(match, "match") and iscallable(getattr(match, "match")):
            # Don't know what type it is, but it has a callable "match"
            # attribute, so we use that.
            matcher = match

        else:
            # Unknown type!
            raise InvalidMatchTypeException("Unknown type: %r" % (match,))

        return Route(matcher, func)

    def add_route(self, method, match, func):
        # Methods are uppercase.
        method = method.upper()

        # Check for valid method.
        if not method in self.SUPPORTED_METHODS:
            raise HobokenException("Invalid method type given: %s" % (method,))

        route = self._make_route(match, func)
        route.method = method
        self.routes[method].append(route)

    def find_route_with_method(self, method, func):
        for route in self.routes[method]:
            if route.func == func:
                return route

        return None

    def find_route(self, func):
        for method in self.SUPPORTED_METHODS:
            route = self.find_route_with_method(method, func)
            if route:
                return route

        return None

    def url_for(self, function, *args, **kwargs):
        route = self.find_route(function)
        if route is None:
            return None

        path = route.reverse(*args, **kwargs)
        return path

    def redirect(self, location, *args, **kwargs):
        """
        This is a helper function for redirection.
        """

        # If a code is specified, we take that.
        code = kwargs.pop('status_code', None)
        if code is None:
            # If no code, we send a 303 if it's supported and we aren't already using GET.
            if self.request.http_version == 'HTTP/1.1' and self.request.method != 'GET':
                code = 303
            else:
                code = 302

        # Re-set the code parameter.
        kwargs['status_code'] = code

        # Set the 'location' argument, which in turn sets the 'Location' header.
        kwargs['location'] = location

        # Halt routing with these parameters.
        halt(*args, **kwargs)

    def _decorate_and_route(self, method, match):
        def internal_decorator(func):
            # We only allow one route for each function.
            if is_route(func):
                raise RouteExistsException()

            # This allows us to add conditions!
            def add_condition(condition_func):
                route = self.find_route(func)
                route.add_condition(condition_func)
                self.logger.debug("Added condition '{0}' for func {1}/{2}:".format(
                    condition_func.__name__, str(method), func.__name__))

            # Add the route.
            self.add_route(method, match, func)

            # Add each of the existing conditions.
            conditions = get_func_attr(func, 'hoboken.conditions', default=[], delete=True)
            for c in conditions:
                add_condition(c)

            # Mark this function as a route.
            set_func_attr(func, 'hoboken.route', True)

            # Add a function to add future conditions. This is so the order
            # of conditions being added doesn't matter.
            set_func_attr(func, 'hoboken.add_condition', add_condition)
            return func
        return internal_decorator

    def add_before_filter(self, match, func):
        filter_tuple = self._make_route(match, func)
        self.before_filters.append(filter_tuple)

    def before(self, match=None):
        # If the match isn't provided, we match anything.
        if match is None:
            match = re.compile(".*")

        def internal_decorator(func):
            self.add_before_filter(match, func)
            return func
        return internal_decorator

    def add_after_filter(self, match, func):
        filter_tuple = self._make_route(match, func)
        self.after_filters.append(filter_tuple)

    def after(self, match=None):
        # If the match isn't provided, we match anything.
        if match is None:
            match = re.compile(".*")

        def internal_decorator(func):
            self.add_after_filter(match, func)
            return func
        return internal_decorator

    def on_returned_body(self, request, resp, value):
        """
        This function is used to turn a value that's been returned from a
        route function into the request body.  Override this in a subclass
        to customize how values are returned.
        """
        if sys.version_info[0] >= 3:        # pragma: no cover
            if isinstance(value, bytes):
                resp.body = value
            elif isinstance(value, str):
                resp.text = value
            else:
                raise ValueError("Unknown return type: {0!r}".format(type(value)))
        else:                               # pragma: no cover
            if isinstance(value, unicode):
                resp.text = value
            else:
                resp.body = value

    def wsgi_entrypoint(self, environ, start_response):
        try:
            # Create our request object.
            self.request = Request(environ)

            # Create an empty response.
            self.response = Response()

            # Actually handle this request.
            self._handle_request()

            # Finally, given our response, we finish the WSGI request.
            return self.response(environ, start_response)
        finally:
            # After each request, we remove the request and response objects.
            del self.request
            del self.response

    def _run_routes(self, method):
        # Since these are thread-locals, we grab them as locals.
        request = self.request
        response = self.response

        # For each route of the specified type, try to match it.
        for route in self.routes[method]:
            matches, ret = route(request, response)
            if ret is not None:
                self.on_returned_body(request, response, ret)

            if matches:
                return True

        return False

    def _handle_request(self):
        # Since these are thread-locals, we grab them as locals.
        request = self.request
        response = self.response
        self.logger.debug("%s %s", request.method, request.url)

        # Check for valid method.
        # TODO: Should this call our after filters?
        if request.method not in self.SUPPORTED_METHODS:
            self.logger.warn("Called with invalid method: %r", request.method)

            # TODO: hook.

            # Send "invalid method" exception.
            response.status_code = 405
            return

        matched = False
        try:
            # Call before filters.
            for filter in self.before_filters:
                filter(request, response)

            # For each route of the specified type, try to match it.
            matched = self._run_routes(request.method)

            # We special-case the HEAD method to fallback to GET.
            if request.method == 'HEAD' and not matched:
                # Run our routes against the 'GET' method.
                matched = self._run_routes('GET')

        except HaltRoutingException as ex:
            # For each attribute in the given kwargs, send it.
            for attr, val in ex.kwargs.items():
                if val is not None:
                    setattr(response, attr, val)

            # Must set this, or we get clobbered by the 404 handler.
            matched = True

        except Exception as e:
            # Also, check if the exception has other information attached,
            # like a code/body.
            # TODO: Handle other HTTPExceptions from webob?
            self.on_exception(e)

            # Must set this, or we get clobbered by the 404 handler.
            matched = True

        finally:
            # Call our after filters
            for route in self.after_filters:
                route(request, response)

        if not matched:
            self.on_route_missing()

    def __call__(self, environ, start_response):
        return self.wsgi_entrypoint(environ, start_response)

    def on_route_missing(self):
        """
        This function is called when a route to handle a request is not found.
        Override this function to provide custom not-found logic.
        """
        # If we have a sub_app, return what it has.
        if self.sub_app:
            self.response = self.request.get_response(self.sub_app)
        else:
            # By default, return a 404 request.
            self.response.status_code = 404

    def on_exception(self, exception):
        self.response.status_code = 500
        if self.debug:
            # Format the current traceback
            tb = traceback.format_exc()

            # Return the traceback as text.
            self.response.content_type = 'text/plain'
            if sys.version_info[0] >= 3:
                self.response.text = tb
            else:
                self.response.body = tb

            print(tb, file=sys.stderr)

    def test_server(self, port=8000):                   # pragma: no cover
        """
        This method lets you start a test server for development purposes.
        Note: There is deliberately no option to set the address to listen on.
              The server will always listen on 'localhost', and should never
              be used in production.
        """

        from wsgiref.simple_server import make_server
        httpd = make_server('localhost', port, self)
        httpd.serve_forever()

    def __str__(self):
        """
        Some help for debugging: repr(app) will get a summary of the app and
        it's defined before/after/routes.
        """

        body = []
        body.append("Application {0} (Debug: {1})".format(self.name, self.debug))
        body.append("")

        def dump_filter_array(arr):
            body.append("=" * 79)
            body.append("Function        Match                     Conditions")
            body.append("-" * 79)
            for filter in arr:
                conds = ", ".join([f.__name__ for f in filter.conditions])
                body.append("{0:<15} {1:<25} {2:<35}".format(filter.func.__name__, str(filter.match), conds))

        def dump_route_array(arr):
            body.append("=" * 79)
            body.append("Method  Function        Match                     Conditions")
            body.append("-" * 79)
            for method in self.routes:
                for route in self.routes[method]:
                    conds = ", ".join([f.__name__ for f in route.conditions])
                    body.append("{0:<7} {1:<15} {2:<25} {3:<28}".format(method, route.func.__name__, str(route.matcher), conds))

        body.append("BEFORE FILTERS")
        dump_filter_array(self.before_filters)
        body.append("")

        body.append("ROUTES")
        dump_route_array(self.before_filters)
        body.append("")

        body.append("AFTER FILTERS")
        dump_filter_array(self.after_filters)
        body.append("")

        return '\n'.join(body)

    def __repr__(self):
        return "HobokenApplication(name={!r}, debug={!r})".format(self.name, self.debug)




from . import HobokenTestCase, skip
from hoboken import condition
from hoboken.conditions import *

import unittest


class TestUserAgentCondition(HobokenTestCase):
    def after_setup(self):
        @condition(user_agent("^Uagent1"))
        @self.app.get("/")
        def route_one():
            return "one"

        @condition(user_agent("^Uagent2"))
        @self.app.get("/")
        def route_two():
            return "two"

    def test_useragent_one(self):
        self.assert_body_is("one", user_agent='Uagent1 v1.2.3')

    def test_useragent_two(self):
        self.assert_body_is("two", user_agent='Uagent2 v4.5.6')


class TestHostCondition(HobokenTestCase):
    def after_setup(self):
        @condition(host("sub1.foobar.com"))
        @self.app.get('/')
        def sub1():
            return 'sub1'

        @condition(host("sub2.foobar.com"))
        @self.app.get('/')
        def sub2():
            return 'sub2'

    def test_host1(self):
        self.assert_body_is("sub1", host='sub1.foobar.com')

    def test_host2(self):
        self.assert_body_is("sub2", host='sub2.foobar.com')


class TestAcceptsCondition(HobokenTestCase):
    def after_setup(self):
        @condition(accepts("text/html"))
        @self.app.get('/')
        def html():
            return 'html'

        @condition(accepts("text/plain"))
        @self.app.get('/')
        def plain():
            return 'plain'

        @condition(accepts("text/*"))
        @self.app.get("/")
        def text():
            return 'text'

        @condition(accepts(["application/json", "application/xml"]))
        @self.app.get("/two")
        def accepts_two():
            return 'two'

        self.app.debug = True

    def test_plain_accept(self):
        self.assert_body_is("html", accepts="text/html")

    def test_compound_accept(self):
        self.assert_body_is("html", accepts="text/html; q=1, application/other; q=0.5")

    def test_other_accept(self):
        self.assert_body_is("plain", accepts="text/plain")

    @skip("Since webob won't match this mime type")
    def test_general_match(self):
        self.assert_body_is("text", "text/otherfoo")

    def test_list_of_accepts(self):
        self.assert_body_is("two", path="/two", accepts="application/json")
        self.assert_body_is("two", path="/two", accepts="application/xml")


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestUserAgentCondition))
    suite.addTest(unittest.makeSuite(TestHostCondition))
    suite.addTest(unittest.makeSuite(TestAcceptsCondition))

    return suite




import re
import sys

__all__ = ["iscallable", "BaseStringType", "StringIO", "RegexType"]

# Determine Python version.
python_version = sys.version_info[:3]
is_python3 = python_version >= (3, 0, 0)

if is_python3:
    import collections

    # callable() replacement
    def iscallable(f):
        return isinstance(f, collections.Callable)

    # basestring
    BaseStringType = str

    # StringIO
    from io import StringIO
else:
    def iscallable(f):
        return callable(f)

    # basestring
    BaseStringType = basestring

    # StringIO
    try:
        from cStringIO import StringIO
    except ImportError:
        from StringIO import StringIO


# Cross-Python stuff.
RegexType = type(re.compile(""))
RegexMatchType = type(re.compile(".*").match("asdf"))



#!/usr/bin/env python

import os
import sys
from setuptools import setup

setup(name='Hoboken',
      version='0.0.1',
      description='A Sinatra-inspired web framework for Python',
      author='Andrew Dunham',
      url='http://github.com/andrew-d/hoboken',
      license='Apache',
      platforms='any',
      zip_safe=False,
      install_requires=[
        'WebOb>=1.2'
      ],
      tests_require=[
          'Mock',
          'PyYAML'
      ],
      packages=['hoboken'],
      classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      test_suite = 'hoboken.tests.suite',
     )




from __future__ import print_function

from . import HobokenTestCase, skip, skip_if, is_pypy
import sys
import time
import unittest
import datetime
from webob import Request
from mock import patch, MagicMock


class TestLastModified(HobokenTestCase):
    def after_setup(self):
        self.time = datetime.datetime(year=2012, month=7, day=15)
        self.app.debug = True

        @self.app.get("/resource")
        def resource():
            self.app.check_last_modified(self.time)
            return b'resource value'

        @self.app.put("/resource")
        def resource():
            self.app.check_last_modified(self.time)
            return b'resource value'

        @self.app.get("/bad")
        def bad_date():
            self.app.check_last_modified(None)
            return b'foo'

    def test_has_last_modified(self):
        r = Request.blank("/resource")
        resp = r.get_response(self.app)
        self.assert_true(resp.last_modified is not None)

    def test_no_last_modified(self):
        r = Request.blank("/bad")
        resp = r.get_response(self.app)
        self.assert_true(resp.last_modified is None)

    def test_no_last_modified_if_etag(self):
        r = Request.blank("/resource")
        r.if_none_match = 'foobar'
        resp = r.get_response(self.app)
        self.assert_true(resp.last_modified is not None)

    def test_will_return_304_for_get(self):
        r = Request.blank("/resource")
        r.if_modified_since = datetime.datetime(year=2012, month=7, day=20)
        resp = r.get_response(self.app)
        self.assert_equal(resp.status_code, 304)
        self.assert_equal(resp.body, b'')

    def test_will_return_200_for_newer(self):
        r = Request.blank("/resource")
        r.if_modified_since = datetime.datetime(year=2012, month=7, day=1)
        resp = r.get_response(self.app)
        self.assert_equal(resp.status_code, 200)
        self.assert_equal(resp.body, b'resource value')

    def test_if_unmodified_since(self):
        r = Request.blank("/resource", method='PUT')
        r.if_unmodified_since = datetime.datetime(year=2012, month=7, day=20)
        r.body = b'put this here'
        resp = r.get_response(self.app)

        self.assert_equal(resp.status_code, 200)

    def test_if_unmodified_since_precondition_fail(self):
        r = Request.blank("/resource", method='PUT')
        r.if_unmodified_since = datetime.datetime(year=2012, month=7, day=1)
        r.body = b'put this here'
        resp = r.get_response(self.app)

        self.assert_equal(resp.status_code, 412)


class TestETag(HobokenTestCase):
    def after_setup(self):
        self.etag = 'some etag'
        self.app.debug = True

        @self.app.get("/resource")
        def resource():
            self.app.check_etag(self.etag)
            return b'resource value'

        @self.app.put("/resource")
        def resource():
            self.app.check_etag(self.etag)
            return b'success'

    def call_app(self, path, *args, **kwargs):
        req = Request.blank(path, *args, **kwargs)
        resp = req.get_response(self.app)
        return resp

    def test_etag(self):
        resp = self.call_app("/resource")
        self.assert_equal(resp.etag, self.etag)

    def test_etag_will_return_304_for_correct_if_none_match(self):
        resp = self.call_app("/resource", if_none_match=self.etag)
        self.assert_equal(resp.status_code, 304)

    def test_etag_will_return_304_for_incorrect_if_none_match(self):
        resp = self.call_app("/resource", method='PUT', if_none_match=self.etag)
        self.assert_equal(resp.status_code, 412)

    def test_etag_will_return_200_for_no_if_none_match(self):
        resp = self.call_app("/resource", method='PUT', if_none_match=self.etag + 'false')
        self.assert_equal(resp.status_code, 200)

    def test_etag_will_return_200_for_correct_if_match(self):
        resp = self.call_app("/resource", method='PUT', if_match=self.etag)
        self.assert_equal(resp.status_code, 200)

    def test_etag_will_return_412_for_incorrect_if_match(self):
        resp = self.call_app("/resource", method='PUT', if_match=self.etag + 'fail')
        self.assert_equal(resp.status_code, 412)

    def test_wildcard_etag_if_match(self):
        resp = self.call_app("/resource", method='PUT', if_match='*')
        self.assert_equal(resp.status_code, 200)

    def test_wildcard_etag_if_none_match(self):
        resp = self.call_app("/resource", method='PUT', if_none_match='*')
        self.assert_equal(resp.status_code, 412)


class TestCacheControl(HobokenTestCase):
    def after_setup(self):
        @self.app.get("/resource")
        def resource():
            self.app.set_cache_control(public=True, no_store=True)
            return b'resource'

    def call_app(self, path, *args, **kwargs):
        req = Request.blank(path, *args, **kwargs)
        resp = req.get_response(self.app)
        return resp

    def test_cache_control(self):
        resp = self.call_app('/resource')
        self.assert_true(resp.cache_control.public)
        self.assert_true(resp.cache_control.no_store)


class TestExpires(HobokenTestCase):
    def after_setup(self):
        @self.app.get("/ccoffset")
        def resource():
            self.app.set_expires(10)
            return b'resource'

        @self.app.get("/ccabsolute")
        def resource():
            time = datetime.datetime(year=2012, month=7, day=15, hour=0, minute=1, second=0)
            self.app.set_expires(time)
            return b'resource'

        @self.app.get("/ccexpired")
        def resource():
            time = datetime.datetime(year=2012, month=7, day=15, hour=0, minute=0, second=0)
            self.app.set_expires(time)
            return b'resource'

    def call_app(self, path, *args, **kwargs):
        req = Request.blank(path, *args, **kwargs)
        resp = req.get_response(self.app)
        return resp

    @skip_if(is_pypy(), "Apparently mocking standard types doesn't work well on PyPy")
    def test_cache_control_with_offset(self):
        test_time = datetime.datetime(year=2012, month=7, day=15, hour=0, minute=0, second=0)
        with patch('datetime.datetime', wraps=datetime.datetime) as new_datetime:
            new_datetime.now.return_value = test_time
            resp = self.call_app("/ccoffset")

        # Note that webob assumes the input time is a UTC value, so we simply
        # remove the timezone info from the expires value to check it.
        expires_timestamp = time.mktime(resp.expires.replace(tzinfo=None).timetuple())
        test_timestamp = time.mktime(test_time.timetuple())
        self.assert_equal(expires_timestamp, test_timestamp + 10)
        self.assert_equal(resp.cache_control.max_age, 10)

    @skip_if(is_pypy(), "Apparently mocking standard types doesn't work well on PyPy")
    def test_cache_control_with_absolute(self):
        test_time = datetime.datetime(year=2012, month=7, day=15, hour=0, minute=0, second=0)
        with patch('datetime.datetime', wraps=datetime.datetime) as new_datetime:
            new_datetime.now.return_value = test_time
            resp = self.call_app("/ccabsolute")

        # Note that webob assumes the input time is a UTC value, so we simply
        # remove the timezone info from the expires value to check it.
        expires_timestamp = time.mktime(resp.expires.replace(tzinfo=None).timetuple())
        test_timestamp = time.mktime(test_time.timetuple())
        self.assert_equal(expires_timestamp, test_timestamp + 60)
        self.assert_equal(resp.cache_control.max_age, 60)

    @skip_if(is_pypy(), "Apparently mocking standard types doesn't work well on PyPy")
    def test_cache_control_already_expired(self):
        test_time = datetime.datetime(year=2012, month=7, day=15, hour=1, minute=0, second=0)
        with patch('datetime.datetime', wraps=datetime.datetime) as new_datetime:
            new_datetime.now.return_value = test_time
            resp = self.call_app("/ccexpired")

        self.assert_equal(resp.cache_control.max_age, 0)


class TestRedirection(HobokenTestCase):
    def after_setup(self):
        @self.app.get("/redirect_back")
        def redirect():
            self.app.redirect_back()

        @self.app.get("/redirect_fail")
        def redirect_fail():
            self.assert_false(self.app.redirect_back())
            return b'no redirect'

        @self.app.get("/to_me")
        def target():
            return b'target'

        @self.app.get("/redirect_to")
        def redirect_to():
            self.app.redirect_to(target)

    def test_redirect_back_success(self):
        r = Request.blank("/redirect_back", referer='http://www.google.com')
        resp = r.get_response(self.app)

        self.assert_true(300 <= resp.status_int <= 399)
        self.assert_equal(resp.location, 'http://www.google.com')

    def test_redirect_back_failure(self):
        r = Request.blank("/redirect_fail", referer='')
        resp = r.get_response(self.app)

        self.assert_equal(resp.status_code, 200)
        self.assert_equal(resp.body, b'no redirect')

    def test_redirect_to(self):
        r = Request.blank("/redirect_to")
        resp = r.get_response(self.app)

        self.assert_true(300 <= resp.status_int <= 399)
        self.assert_true(resp.location.endswith('/to_me'))


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestLastModified))
    suite.addTest(unittest.makeSuite(TestETag))
    suite.addTest(unittest.makeSuite(TestCacheControl))
    suite.addTest(unittest.makeSuite(TestExpires))
    suite.addTest(unittest.makeSuite(TestRedirection))

    return suite

